import pandas as pd
import numpy as np
from optparse import OptionParser

parser = OptionParser()
parser.add_option("-f", dest = "fold", default = "", help = "Welches Fold wird benutzt")
options, args = parser.parse_args()


label = 1               # class for which the data is to be augmented
method = 'MWO'          # augmentation method
f = int(options.fold)   # fold from cross validation
augFactor = 5           # augmentation factor


################################################################
###################### LOAD ORIGINAL DATA ######################
################################################################

# Load file including training and test data for the specific k-fold split 
# Besides gene expression values data set has to contain information about 
# the subset type ('train' or 'test') as well as the labels of the samples (0 or 1)
DF = pd.read_csv('/home/magdalena/PhD/2021_12/GSE163151/train_test_splits/train_test_split_fold{}.csv'.format(f), index_col=0)

# only include training data from specified class (label)
mwodata = DF[(DF.subset == 'train') & (DF.label == label)]


###################################################
###################### MODEL ######################
###################################################

def MWO(data):
	X = train.select_dtypes('float')
	label = train.label.unique().item()
	genes = X.columns.tolist()
	n, p = X.shape
	nnew = n * augFactor
	mX = X.mean(axis=0)
	sdX = X.std(axis=0)
	idxnames = ['a{}_label{}'.format(i+1, label) for i in range(nnew)]
	colnames = X.columns.tolist()
	Z0 = pd.DataFrame(index=idxnames, columns=colnames)
	for i in range(nnew):
		w = np.concatenate((0, np.sort(np.random.uniform(size=n-1)), 1), axis=None)
		w = w[1:(n+1)] - w[0:n]
		Xt = X.sample(frac=1, replace=True).T
		Z0.iloc[i] = Xt.dot(w)
	mZ0 = Z0.mean(axis=0)
	sdZ0 = Z0.std(axis=0)
	for i in range(len(Z0)):
		Z0.iloc[i] = ((((Z0.iloc[i] - mZ0) / (sdZ0 + 1e-5)) * sdX) + mX)
	Z0.insert(0, 'label', [label]*nnew)
	Z0.insert(0, 'id', idxnames)
	Z0.insert(0, 'subset', ['train']*nnew)
	columns = ['subset','id','label'] + genes
	aug_df = data[columns].append(Z0[columns])
	aug_df[genes] = aug_df[genes].astype('float')
	return aug_df

# create augmented dataframe containing artificial and original training samples and save as csv file
aug_df = MWO(mwodata)
aug_df.to_csv('{}_SubsetLabel{}_5times_augmented_train_test_fold{}.csv'.format(method, label, f), index=False)